/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "ginfo.h"
#include "menu.h"

static GtkWidget *menu_prefwin;
static GtkWidget *menu_color_button;
static GtkWidget *menu_radiobutton;
static GtkWidget *menu_csd;
static GtkWidget *menu_selection;
static char *menu_new_color = NULL;
static GtkWidget *menu_spinbutton;

void menu_change_color (GtkWidget *, gpointer);
void menu_color (GtkWidget *, gpointer);
void menu_color_close (GtkWidget *, gpointer);
void menu_cancel_preference (GtkWidget *, gpointer);
void menu_save_preference (GtkWidget *, gpointer);
void menu_set_modules (GtkWidget *);
void menu_dialog (char *, ...);
int menu_ask_delete(void);
void menu_modules_delete(GtkWidget *, GtkTreeSelection *);
int menu_modules_check_number(void);
void menu_modules_close(GtkWidget *, GtkWidget *);

void
menu_preference (GtkWidget * w, gpointer dummy)
{
  GtkWidget *frame;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *label;
  GtkWidget *button;
  GtkWidget *sep;
  GtkWidget *box;
  GtkWidget *radiobutton;
  GSList *radiobutton_group = NULL;
  GtkObject *spinbutton_adj;
  char buf[1024];
  GdkColor color;

  snprintf (buf, 1024, "%s %s - Preference", PACKAGE, VERSION);

  menu_prefwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (menu_prefwin), buf);
  gtk_window_set_modal (GTK_WINDOW (menu_prefwin), TRUE);

  g_signal_connect ((gpointer) menu_prefwin, "destroy",
		    G_CALLBACK (menu_cancel_preference), NULL);

  frame = gtk_frame_new (buf);
  gtk_widget_show (frame);
  gtk_container_add (GTK_CONTAINER (menu_prefwin), frame);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (frame), vbox);

  hbox = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);

  label = gtk_label_new ("Border color:");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  menu_color_button = gtk_button_new_with_mnemonic ("Color");
  gtk_widget_show (menu_color_button);
  gtk_box_pack_start (GTK_BOX (hbox), menu_color_button, TRUE, TRUE, 0);

  if (!menu_new_color)
    menu_new_color = strdup (graphic_color);
  gdk_color_parse (menu_new_color, &color);

  gtk_widget_modify_bg (menu_color_button, GTK_STATE_NORMAL, &color);

  g_signal_connect ((gpointer) menu_color_button, "clicked",
		    G_CALLBACK (menu_color), NULL);

  sep = gtk_hseparator_new ();
  gtk_widget_show (sep);
  gtk_box_pack_start (GTK_BOX (vbox), sep, FALSE, TRUE, 0);

  hbox = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);

  label = gtk_label_new ("Tips:");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  box = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (box);
  gtk_box_pack_start (GTK_BOX (hbox), box, TRUE, TRUE, 0);

  menu_radiobutton = gtk_radio_button_new_with_mnemonic (NULL, "yes");
  gtk_widget_show (menu_radiobutton);
  gtk_box_pack_start (GTK_BOX (box), menu_radiobutton, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (menu_radiobutton),
			      radiobutton_group);
  radiobutton_group =
    gtk_radio_button_get_group (GTK_RADIO_BUTTON (menu_radiobutton));

  radiobutton = gtk_radio_button_new_with_mnemonic (NULL, "no");
  gtk_widget_show (radiobutton);
  gtk_box_pack_start (GTK_BOX (box), radiobutton, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton),
			      radiobutton_group);
  radiobutton_group =
    gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton));

  if (tooltip_done)
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), TRUE);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (menu_radiobutton),
				    FALSE);
    }
  else
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton), FALSE);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (menu_radiobutton),
				    TRUE);
    }

  sep = gtk_hseparator_new ();
  gtk_widget_show (sep);
  gtk_box_pack_start (GTK_BOX (vbox), sep, FALSE, TRUE, 0);

  hbox = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

  label = gtk_label_new ("Time tips:");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  spinbutton_adj = gtk_adjustment_new (tooltip_time, 1, 100, 1, 10, 10);
  menu_spinbutton =
    gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_adj), 1, 0);
  gtk_widget_show (menu_spinbutton);
  gtk_box_pack_start (GTK_BOX (hbox), menu_spinbutton, TRUE, TRUE, 0);

  sep = gtk_hseparator_new ();
  gtk_widget_show (sep);
  gtk_box_pack_start (GTK_BOX (vbox), sep, FALSE, TRUE, 0);

  hbox = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

  button = gtk_button_new_with_mnemonic ("_Cancel");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked",
		    G_CALLBACK (menu_cancel_preference), NULL);

  button = gtk_button_new_with_mnemonic ("_Save");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked",
		    G_CALLBACK (menu_save_preference), NULL);

  gtk_widget_show (menu_prefwin);
}

void
menu_editor (GtkWidget * w, gpointer dummy)
{
  char *a[2];
  pid_t pid;

  a[0] = GINFO_EDITOR;
  a[1] = NULL;

  if (!fork ())
    {
      if (!(pid = fork ()))
	{
	  execvp (a[0], a);
	  exit (0);
	}
      else
	exit (0);
    }
  else
    wait (NULL);
}

void
menu_about (GtkWidget * w, gpointer dummy)
{
  GtkWidget *dialog;
  char buf[1024];

  snprintf (buf, 1024, "%s %s\n\nby %s\n%s", PACKAGE, VERSION, AUTHOR,
	    COPYRIGHT);

  dialog =
    gtk_message_dialog_new (GTK_WINDOW (mainwin),
			    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			    GTK_MESSAGE_INFO, GTK_BUTTONS_OK, buf);

  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);

}

void
menu_quit (GtkWidget * w, gpointer dummy)
{
  GtkWidget *dialog;
  int ret;

  dialog =
    gtk_message_dialog_new (GTK_WINDOW (mainwin),
			    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			    GTK_MESSAGE_INFO, GTK_BUTTONS_OK_CANCEL,
			    "You are sure to quit?");

  ret = gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);

  if (ret == GTK_RESPONSE_OK)
    {
      gtk_widget_hide (mainwin);
      gtk_widget_destroy (mainwin);
      gtk_main_quit ();
    }
}

void
menu_color (GtkWidget * w, gpointer dummy)
{
  GtkWidget *button;
  GdkColor color;
  char buf[1024];

  snprintf (buf, 1024, "%s %s - Select Color", PACKAGE, VERSION);

  menu_csd = gtk_color_selection_dialog_new (buf);
  gtk_window_set_resizable (GTK_WINDOW (menu_csd), FALSE);
  gtk_window_set_modal (GTK_WINDOW (menu_csd), TRUE);

  button = GTK_COLOR_SELECTION_DIALOG (menu_csd)->ok_button;
  gtk_widget_show (button);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) button, "clicked",
		    G_CALLBACK (menu_change_color), NULL);

  button = GTK_COLOR_SELECTION_DIALOG (menu_csd)->cancel_button;
  gtk_widget_show (button);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) button, "clicked",
		    G_CALLBACK (menu_color_close), NULL);

  button = GTK_COLOR_SELECTION_DIALOG (menu_csd)->help_button;
  gtk_widget_show (button);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  if (!menu_new_color)
    menu_new_color = strdup (graphic_color);
  gdk_color_parse (menu_new_color, &color);

  menu_selection = GTK_COLOR_SELECTION_DIALOG (menu_csd)->colorsel;
  gtk_widget_show (menu_selection);
  gtk_color_selection_set_has_opacity_control (GTK_COLOR_SELECTION
					       (menu_selection), FALSE);
  gtk_color_selection_set_current_color (GTK_COLOR_SELECTION (menu_selection),
					 &color);

  gtk_widget_show (menu_csd);
}

void
menu_color_close (GtkWidget * w, gpointer dummy)
{
  gtk_widget_destroy (menu_csd);
}

void
menu_change_color (GtkWidget * w, gpointer dummy)
{
  GdkColor color;
  unsigned char red, blue, green;
  char *a;

  gtk_color_selection_get_current_color (GTK_COLOR_SELECTION (menu_selection),
					 &color);

  red = (color.red >> 8);
  blue = (color.blue >> 8);
  green = (color.green >> 8);

  if (!(a = (char *) malloc (sizeof (char) * 8)))
    fatal ("Error: memory.");
  snprintf (a, 8, "#%.2X%.2X%.2X", red, green, blue);

  if (menu_new_color)
    free (menu_new_color);

  menu_new_color = a;
  gtk_widget_modify_bg (menu_color_button, GTK_STATE_NORMAL, &color);

  gtk_widget_destroy (menu_csd);
}

void
menu_cancel_preference (GtkWidget * w, gpointer dummy)
{
  if (menu_new_color)
    free (menu_new_color);

  menu_new_color = NULL;

  gtk_widget_destroy (menu_prefwin);
}

void
menu_save_preference (GtkWidget * w, gpointer dummy)
{
  GdkColor color;

  /* BORDER COLOR */
  if (menu_new_color)
    {
      if (graphic_color)
	free (graphic_color);

      graphic_color = menu_new_color;
    }

  menu_new_color = NULL;

  /* TIPS? */
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (menu_radiobutton)) ==
      TRUE)
    tooltip_done = 0;
  else
    tooltip_done = 1;

  /* TIME */
  tooltip_time =
    (int) gtk_spin_button_get_value (GTK_SPIN_BUTTON (menu_spinbutton));
  tooltip_change_time = 1;

  gdk_color_parse (graphic_color, &color);
  gtk_widget_modify_bg (graphic_frame, GTK_STATE_NORMAL, &color);

  gtk_widget_destroy (menu_prefwin);

  config_write ();
}

void
menu_nt (GtkWidget * w, gpointer dummy)
{
  tooltip_request = 1;
  tooltips_timeout (NULL);
}

void
menu_modules (GtkWidget * w, gpointer dummy)
{
  GtkWidget *window;
  GtkWidget *frame;
  GtkWidget *box;
  GtkWidget *scrolledwindow;
  GtkWidget *button;
  GtkListStore *model;
  GtkCellRenderer *renderer;
  GtkWidget *list;
  GtkTreeSelection *selection;

  char buf[255];

  snprintf (buf, 255, "%s %s - modules", PACKAGE, VERSION);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), buf);
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);

  g_signal_connect ((gpointer) window, "destroy",
		    G_CALLBACK (gtk_widget_destroy), NULL);

  frame = gtk_frame_new ("Modules");
  gtk_widget_show (frame);
  gtk_container_add (GTK_CONTAINER (window), frame);

  box = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(box);
  gtk_container_add(GTK_CONTAINER(frame), box);

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow);
  gtk_box_pack_start(GTK_BOX(box), scrolledwindow, FALSE, FALSE, 0);
  gtk_widget_set_size_request (scrolledwindow, 250, -1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  model = gtk_list_store_new (1, G_TYPE_STRING);
  list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
  gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (list), TRUE);
  gtk_tree_view_set_search_column (GTK_TREE_VIEW (list), 0);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (list), -1,
					       "Modules", renderer, "text", 0,
					       NULL);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (list), FALSE);

  gtk_widget_show (list);
  g_object_unref (model);

  gtk_container_add (GTK_CONTAINER (scrolledwindow), list);

  menu_set_modules (list);

  button = gtk_button_new_with_mnemonic("_Delete");
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (menu_modules_delete), selection);

  button = gtk_button_new_with_mnemonic("_Close");
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (menu_modules_close), window);

  gtk_widget_show (window);
}

void menu_modules_close(GtkWidget *w, GtkWidget *win) {

  if(!win) return;

  gtk_widget_hide(win);
  gtk_widget_destroy(win);
}

void menu_modules_delete(GtkWidget *w, GtkTreeSelection *selection) {
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkTreePath *path;
  int k;

  if(!selection) return;

  tutor_close(NULL,NULL);

  if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
  GtkWidget *dialog;

  if(menu_modules_check_number()) return;

  dialog =
    gtk_message_dialog_new (GTK_WINDOW (mainwin),
			    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			    GTK_MESSAGE_INFO, GTK_BUTTONS_OK_CANCEL,
			    "Sure to delete this module?");

    if(gtk_dialog_run(GTK_DIALOG(dialog))== GTK_RESPONSE_OK) {
      module *m=module_head;

      path=gtk_tree_model_get_path(model, &iter);
      k=gtk_tree_path_get_indices(path)[0];
      gtk_tree_path_free(path);

      while(m) {

        if(!k) {
	    int type;
	  if(!strncmp(m->file+strlen(m->file)-3, ".so",3)) 
	    type=GINFO_C;
	  else  type=GINFO_XML;

          gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
	  modules_delete(m->file, type);
	}

	k--;
        m=m->next;
      }
    }

    gtk_widget_destroy(dialog);
  }
}

int menu_modules_check_number(void) {
  module *m=module_head;
  int k=0;

  while(m) {
    k++;
    m=m->next;
  }

  if(k<2) {
    menu_dialog("I can't live with 0 module :/");
    return 1;
  }

  return 0;
}

void
menu_set_modules (GtkWidget * list)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  module *t;
  char buf[1024];

  model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));

  t = module_head;
  while (t)
    {
      gtk_list_store_append (GTK_LIST_STORE (model), &iter);

      snprintf (buf, 1024, "%s: %s", t->name, t->file);
      gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, buf, -1);

      t = t->next;
    }
}

/* Add modules */
void
menu_add_module (GtkWidget * w, gpointer dummy)
{
  GtkWidget *dialog;

  dialog =
    gtk_file_chooser_dialog_new ("Add XML module...", GTK_WINDOW (mainwin),
				 GTK_FILE_CHOOSER_ACTION_OPEN,
				 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				 GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);

  gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dialog), FALSE);

  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    {
      char *module =
	gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      struct stat st;
      char *tmp;
      int ret=0;
      int type;

      if (!module)
	menu_dialog ("No modules?");
      else
	{

	  if (!lstat (module, &st))
	    {
	      tmp = module;
	    }
	  else
	    {
	      int n;

	      n = strlen (GINFO_XML_PATH_MODULES) + strlen (module) + 2;

	      if (!(tmp = (char *) malloc (sizeof (char) * n)))
		fatal ("Error: memory.");

	      snprintf (tmp, n, "%s/%s", GINFO_XML_PATH_MODULES, module);

	      if (lstat (tmp, &st))
		{
		  menu_dialog ("I can't open your module!");
		  free (tmp);
		  tmp = NULL;
		}
	    }

	  if (tmp)
	    {
	      if(!strcmp(tmp+strlen(tmp)-3,".so")) type=GINFO_C;
	      else type=GINFO_XML;

	      if(modules_delete_search(tmp) && (ret=menu_ask_delete())==1) {
	        modules_delete(tmp, type);
	      }

              if(ret!=-1) {
	        if(type==GINFO_XML)
  	          modules_xml_check (tmp);
		else
  	          modules_c_check (tmp);
		}

	      if (tmp != module)
		free (tmp);
	    }
	}
    }

  gtk_widget_destroy (dialog);
}

void
menu_add_remote (GtkWidget * w, gpointer dummy)
{
  GtkWidget *dialog;
  GtkWidget *box;
  GtkWidget *label;
  GtkWidget *entry;

  dialog =
    gtk_dialog_new_with_buttons ("Add remote module...", GTK_WINDOW (mainwin),
				 GTK_DIALOG_MODAL |
				 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK,
				 GTK_RESPONSE_OK, GTK_STOCK_CANCEL,
				 GTK_RESPONSE_CANCEL, NULL);

  box = gtk_hbox_new (FALSE, 5);
  gtk_widget_show (box);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), box, FALSE, FALSE,
		      0);

  label = gtk_label_new ("Remote XML Module: ");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

  entry = gtk_entry_new ();
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (box), entry, FALSE, FALSE, 0);

  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
    {
      char *module = (char *) gtk_entry_get_text (GTK_ENTRY (entry));
      char *file;

      if (!module)
	menu_dialog ("No modules?");
      else
	{
	  modules_xml_delete ();

	  if (!(file = modules_xml_download (module)))
	    {
	      menu_dialog ("I can't download your module: %s", file);
	    }
	  else
	    {
	      modules_xml_server_check_list (file,
					     modules_xml_server_check_name
					     (module));

	      if (modules_xml_dir (modules_xml_tmp_dir(GINFO_TMP_DIR)))
		{
		  menu_dialog ("Error: i can't use your remote modules!");
		}

	      modules_xml_delete ();
	    }
	}
    }

  gtk_widget_destroy (dialog);
}

/* Dialog */
void
menu_dialog (char *a, ...)
{
  GtkWidget *d;
  va_list va;
  char buf[1024];

  va_start (va, a);
  vsnprintf (buf, 1024, a, va);

  d = gtk_message_dialog_new (GTK_WINDOW (mainwin),
			      GTK_DIALOG_MODAL |
			      GTK_DIALOG_DESTROY_WITH_PARENT,
			      GTK_MESSAGE_INFO, GTK_BUTTONS_OK, buf);

  gtk_dialog_run (GTK_DIALOG (d));
  gtk_widget_destroy (d);

  va_end (va);
}

int menu_ask_delete(void) {
  GtkWidget *dialog;
  GtkWidget *label;
  int ret=0;

  dialog = gtk_dialog_new_with_buttons("Delete XML module...", GTK_WINDOW(mainwin), GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_YES, GTK_RESPONSE_YES, GTK_STOCK_NO, GTK_RESPONSE_NO, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,NULL);

  label = gtk_label_new("Found a XML module with the same name. Delete the older?");
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 0);

  switch(gtk_dialog_run(GTK_DIALOG(dialog))) {
    case GTK_RESPONSE_YES: ret=1; break;

    case GTK_RESPONSE_CANCEL: ret=-1; break;
  }

  gtk_widget_destroy(dialog);

  return ret;
}

/* EOF */
